<?php

if ( ! defined( 'ABSPATH' ) )
	exit;



/**
 *  wpuxss_eml_pro_prepare_attachment_for_js
 *
 *  @since    2.0
 *  @created  30/07/14
 */

add_filter( 'wp_prepare_attachment_for_js', 'wpuxss_eml_pro_prepare_attachment_for_js', 10, 3 );

if ( ! function_exists( 'wpuxss_eml_pro_prepare_attachment_for_js' ) ) {

    function wpuxss_eml_pro_prepare_attachment_for_js( $response, $attachment, $meta ) {

        foreach ( get_object_taxonomies ( 'attachment', 'names' ) as $taxonomy ) {

            $term_ids = wp_get_object_terms( $attachment->ID, $taxonomy, array( 'fields' => 'ids' ) );
            $response['taxonomies'][$taxonomy] = $term_ids;
        }

        return $response;
    }
}



/**
 *  wpuxss_eml_pro_add_attachment
 *
 *  @since    2.2
 *  @created  11/02/16
 */

add_action( 'add_attachment', 'wpuxss_eml_pro_add_attachment' );

if ( ! function_exists( 'wpuxss_eml_pro_add_attachment' ) ) {

    function wpuxss_eml_pro_add_attachment( $id ) {

        $attachment = get_post( $id );

        if ( ! $attachment->post_parent )
            return;

        $post = get_post( $attachment->post_parent );
        $wpuxss_eml_taxonomies = get_option('wpuxss_eml_taxonomies');

        foreach ( get_object_taxonomies( $post->post_type, 'names' ) as $taxonomy ) {

            if ( ! isset( $wpuxss_eml_taxonomies[$taxonomy] ) ||
                 $wpuxss_eml_taxonomies[$taxonomy]['eml_media'] ||
                 ! $wpuxss_eml_taxonomies[$taxonomy]['assigned'] ||
                 ! $wpuxss_eml_taxonomies[$taxonomy]['taxonomy_auto_assign'] ) {
                continue;
            }

            $term_ids = wp_get_object_terms( $post->ID, $taxonomy, array( 'fields' => 'ids' ) );

            if ( is_wp_error( $term_ids ) || empty( $term_ids ) )
                continue;

            wp_set_object_terms( $id, $term_ids, $taxonomy, false );
        }
    }
}



/**
 *  wpuxss_eml_pro_synchronize_terms
 *
 *  @since    2.2
 *  @created  21/02/16
 */

add_action( 'wp_ajax_eml-synchronize-terms', 'wpuxss_eml_pro_synchronize_terms' );

if ( ! function_exists( 'wpuxss_eml_pro_synchronize_terms' ) ) {

    function wpuxss_eml_pro_synchronize_terms() {

        if ( ! isset( $_REQUEST['post_type'] ) )
            wp_send_json_error();

        if ( ! isset( $_REQUEST['taxonomy'] ) )
            wp_send_json_error();

        check_ajax_referer( 'eml-bulk-edit-nonce', 'nonce' );

        $args = array(
            'posts_per_page' => -1,
            'post_type'      => $_REQUEST['post_type'],
            'post_status'    => 'publish',
        );


        foreach( get_posts( $args ) as $post ) {

            $attachments = get_attached_media( '', $post->ID );

            if ( empty( $attachments ) )
                continue;

            $term_ids = wp_get_object_terms( $post->ID, $_REQUEST['taxonomy'], array( 'fields' => 'ids' ) );

            if ( is_wp_error( $term_ids ) || empty( $term_ids ) )
                continue;

            foreach( $attachments as $attachment ) {

                wp_set_object_terms( $attachment->ID, $term_ids, $_REQUEST['taxonomy'], false );
            }
        }

        wp_send_json_success();
    }
}



/**
 *  wpuxss_eml_pro_pre_get_posts
 * 
 *  Filters whether an attachment query should include filenames or not.
 * 
 *  @since    2.8.11
 *  @created  2024/04
 */

add_action( 'pre_get_posts', 'wpuxss_eml_pro_pre_get_posts' );

if ( ! function_exists( 'wpuxss_eml_pro_pre_get_posts' ) ) {

    function wpuxss_eml_pro_pre_get_posts( $wp_query ) {

        if ( ! $wp_query->is_admin ||
             ! $wp_query->get('s') ||
             'attachment' !== $wp_query->get('post_type') ) {

            return;
        }


        $wpuxss_eml_lib_options = get_option( 'wpuxss_eml_lib_options', array() );


        // it's true for the attachments search by default in the core
        if ( ! in_array( 'filenames', $wpuxss_eml_lib_options['search_in'] ) ) {
            add_filter( 'wp_allow_query_attachment_by_filename', '__return_false' );
        }
    }
}



/**
 *  wpuxss_eml_pro_search_columns
 * 
 *  Filters whether an attachment query should look into titles, captions, descriptions
 * 
 *  @since    2.8.11
 *  @created  2024/04
 */

add_filter( 'post_search_columns', 'wpuxss_eml_pro_search_columns', 10, 3 );

if ( ! function_exists( 'wpuxss_eml_pro_search_columns' ) ) {

    function wpuxss_eml_pro_search_columns( $search_columns, $search_string, $wp_query ) {

        if ( ! $wp_query->is_admin ||
             'attachment' !== $wp_query->get('post_type') ) {

            return $search_columns;
        }


        $wpuxss_eml_lib_options = get_option( 'wpuxss_eml_lib_options', array() );
        $columns_to_keep      = $wpuxss_eml_lib_options['search_in'];


        $columns_to_keep = array_intersect( 
            array(
                'post_title'   => 'titles',
                'post_excerpt' => 'captions',
                'post_content' => 'descriptions'
            ), 
            $columns_to_keep 
        );

        $columns_to_keep = array_flip( $columns_to_keep );
        $search_columns  = array_intersect( $search_columns, $columns_to_keep );

        
        // just in case eml options are somehow empty in the DB  
        if ( empty( $search_columns ) ) {
            $search_columns[] = 'post_title';
        }

        return $search_columns;
    }
}



/**
 *  wpuxss_eml_pro_posts_join
 * 
 *  JOIN clause of the query
 * 
 *  Joins DB tables required for search
 *
 *  @since    2.6
 *  @created  08/03/18
 */

add_filter( 'posts_join', 'wpuxss_eml_pro_posts_join', 10, 2 );

if ( ! function_exists( 'wpuxss_eml_pro_posts_join' ) ) {

    function wpuxss_eml_pro_posts_join( $join, $wp_query ) {

        global $wpdb;


        if ( ! $wp_query->is_admin ||
             ! $wp_query->get('s') ||
             'attachment' !== $wp_query->get('post_type') ) {

            return $join;
        }


        $wpuxss_eml_lib_options = get_option( 'wpuxss_eml_lib_options', array() );


        if ( in_array( 'authors', $wpuxss_eml_lib_options['search_in'] ) ) {
            $join .= "
                LEFT JOIN
                    {$wpdb->users} users ON ( {$wpdb->posts}.post_author = users.ID )
                ";
        }

        if ( in_array( 'taxonomies', $wpuxss_eml_lib_options['search_in'] ) ) {

            $join .= "
                LEFT JOIN
                    {$wpdb->term_relationships} tr ON ( {$wpdb->posts}.ID = tr.object_id )
                LEFT JOIN
                    {$wpdb->term_taxonomy} tt ON ( tr.term_taxonomy_id = tt.term_taxonomy_id )
                LEFT JOIN
                    {$wpdb->terms} t ON ( tt.term_id = t.term_id )
                ";
        }

        return $join;
    }
}



/**
 *  wpuxss_eml_pro_posts_groupby
 * 
 *  Ensures groupby is set for author name and/or taxonomies search
 *
 *  @since    2.8.11
 *  @created  2024/04
 */

add_filter( 'posts_groupby', 'wpuxss_eml_pro_posts_groupby', 10, 2 );

if ( ! function_exists( 'wpuxss_eml_pro_posts_groupby' ) ) {

    function wpuxss_eml_pro_posts_groupby( $groupby, $wp_query ) {

        global $wpdb;


        if ( ! $wp_query->is_admin ||
                 ! $wp_query->get('s') ||
                 'attachment' !== $wp_query->get('post_type') ) {

                return $groupby;
            }


        $wpuxss_eml_lib_options = get_option( 'wpuxss_eml_lib_options', array() );

        if ( in_array( 'authors', $wpuxss_eml_lib_options['search_in'] ) ||
             in_array( 'taxonomies', $wpuxss_eml_lib_options['search_in'] ) 
            ) {
            $groupby = "{$wpdb->posts}.ID";
        }

        return $groupby;
    }
}





/**
 *  wpuxss_eml_pro_posts_search
 * 
 *  WHERE clause of the search query
 * 
 *  Impostant to modify the WHERE clause exactly with the posts_search filter
 *  because we have to replace the latest ) for all LIKE-s
 *  which is produced by parse_search()
 *
 *  @since    2.6
 *  @since    2.8.11 modified 
 * 
 *  @created  08/03/18
 */

add_filter( 'posts_search', 'wpuxss_eml_pro_posts_search', 10, 2 );

if ( ! function_exists( 'wpuxss_eml_pro_posts_search' ) ) {

    function wpuxss_eml_pro_posts_search( $where, $wp_query ) {

        global $wpdb;


        if ( ! $wp_query->is_admin ||
             ! $wp_query->get('s') ||
             'attachment' !== $wp_query->get('post_type') ) {

            return $where;
        }


        $wpuxss_eml_lib_options = get_option( 'wpuxss_eml_lib_options', array() );
        $s = '%' . $wpdb->esc_like( $wp_query->get('s') ) . '%';
        $ext_where = '';


        if ( in_array( 'authors', $wpuxss_eml_lib_options['search_in'] ) ) {
            $ext_where .= $wpdb->prepare( " OR (users.display_name LIKE '%s')", $s );
        }

        if ( in_array( 'taxonomies', $wpuxss_eml_lib_options['search_in'] ) ) {

            $processed_taxonomies = get_object_taxonomies( 'attachment', 'names' );

            foreach( $processed_taxonomies as $taxonomy ) {
                $ext_where .= $wpdb->prepare( " OR (tt.taxonomy = '%s' AND t.name LIKE '%s')", $taxonomy, $s );
            }
        }

        if ( ! empty( $ext_where ) ) {

            $closing = strrpos( $where, ')' );

            if ( $closing ) {
                $where = substr_replace( $where, $ext_where . ' )', $closing, 1 );
            }
            else {
                $where .= $ext_where . ' ';
            }
        }

        return $where;
    }
}
